package cv11;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class TaskDAOImpl implements TaskDAO {

    private static final String SQL_INSERT = "INSERT INTO TASKS (TITLE, PRIORITY, DONE) VALUES (?, ?, ?)";

    private static final String SQL_UPDATE = "UPDATE TASKS SET TITLE = ?, PRIORITY = ?, DONE = ? WHERE ID = ?";

    private static final String SQL_DELETE = "DELETE FROM TASKS WHERE ID = ?";

    private static final String SQL_SELECT_ALL = "SELECT ID, TITLE, PRIORITY, DONE FROM TASKS";

    // DONE je v modelu sice jako BOOLEAN, ale v DB jako INT, proto musíme porovnávat proti číslu
    private static final String SQL_SELECT_OPEN = SQL_SELECT_ALL + " WHERE DONE = 0 AND PRIORITY >= 3";

    private final Connection connection;

    public TaskDAOImpl(Connection connection) {
        this.connection = connection;
    }

    @Override
    public Task insert(Task task) throws SQLException {
        PreparedStatement statement = connection.prepareStatement(SQL_INSERT, Statement.RETURN_GENERATED_KEYS);

        statement.setString(1, task.getTitle());
        statement.setInt(2, task.getPriority());
        statement.setBoolean(3, task.isDone());

        // kontrola vysledku dotazu + vygenerovani klice
        if (statement.executeUpdate() != 1 || !statement.getGeneratedKeys().next()) {
            throw new SQLException("Insert failed");
        }

        Long id = statement.getGeneratedKeys().getLong(1);
        task.setId(id);

        return task;
    }

    @Override
    public Task update(Task task) throws SQLException {
        if (task.getId() == null) {
            throw new IllegalArgumentException("ID is null");
        }

        PreparedStatement statement = connection.prepareStatement(SQL_UPDATE);

        statement.setString(1, task.getTitle());
        statement.setInt(2, task.getPriority());
        statement.setBoolean(3, task.isDone());

        statement.setLong(4, task.getId());

        if (statement.executeUpdate() != 1) {
            throw new SQLException("Update failed");
        }

        return task;
    }

    @Override
    public void delete(Task task) throws SQLException {
        if (task.getId() == null) {
            throw new IllegalArgumentException("ID is null");
        }

        PreparedStatement statement = connection.prepareStatement(SQL_DELETE);

        statement.setLong(1, task.getId());

        if (statement.executeUpdate() != 1) {
            throw new SQLException("Delete failed");
        }
    }

    @Override
    public List<Task> findAll() throws SQLException {

        PreparedStatement statement = connection.prepareStatement(SQL_SELECT_ALL);

        // ziskat pohled z databaze
        ResultSet result = statement.executeQuery();

        // zpracovat result set a prevest ho na kolekci objektu
        return readResult(result);
    }

    @Override
    public List<Task> findOpened() throws SQLException {
        PreparedStatement statement = connection.prepareStatement(SQL_SELECT_OPEN);

        // ziskat pohled z databaze
        ResultSet result = statement.executeQuery();

        // zpracovat result set a prevest ho na kolekci objektu
        return readResult(result);
    }

    private List<Task> readResult(ResultSet result) throws SQLException {
        // sada nalezenych tasku
        List<Task> tasks = new ArrayList<Task>();

        while (result.next()) {

            tasks.add(new Task(
                    result.getLong("ID"),
                    result.getString("TITLE"),
                    result.getInt("PRIORITY"),
                    result.getBoolean("DONE")
            ));
        }

        return tasks;
    }
}
